>From 868829187fe5e735795dd2575793fefa1a2e7d7b Mon Sep 17 00:00:00 2001
From: Arnaud <arnaud.champion@devatom.fr>
Date: Sat, 13 Nov 2010 16:18:44 +0100
Subject: [PATCH] Correct virDomainBlockStats, virDomainInterfaceStats and NodeInfo marshaling

---
 Domain.cs |   52 +++++++++++++++++++++++++++++++++++++++++++++++++---
 Types.cs  |   43 ++++++++++++++++++++++++++++++++-----------
 2 files changed, 81 insertions(+), 14 deletions(-)

diff --git a/Domain.cs b/Domain.cs
index 4f1aba2..1acab58 100644
--- a/Domain.cs
+++ b/Domain.cs
@@ -45,7 +45,25 @@ namespace Libvirt
         /// <param name="size">size of stats structure</param>
         /// <returns>0 in case of success or -1 in case of failure.</returns>
         [DllImport("libvirt-0.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "virDomainBlockStats")]
-        public static extern int BlockStats(IntPtr dom, string path, DomainBlockStatsStruct stats, int size);
+        private static extern int BlockStats(IntPtr dom, string path, IntPtr stats, int size);
+        /// <summary>
+        /// This function returns block device (disk) stats for block devices attached to the domain. The path parameter is the name of the block device. Get this by calling virDomainGetXMLDesc and finding the target dev='...' attribute within //domain/devices/disk. (For example, "xvda"). Domains may have more than one block device. To get stats for each you should make multiple calls to this function. Individual fields within the stats structure may be returned as -1, which indicates that the hypervisor does not support that particular statistic.
+        /// </summary>
+        /// <param name="dom">pointer to the domain object</param>
+        /// <param name="path">path to the block device</param>
+        /// <param name="stats">block device stats (returned)</param>
+        /// <returns>0 in case of success or -1 in case of failure.</returns>
+        public static int BlockStats(IntPtr dom, string path, out DomainBlockStatsStruct stats)
+        {
+            DomainBlockStatsStruct statStruct = new DomainBlockStatsStruct();
+            IntPtr statStructPtr = Marshal.AllocHGlobal(Marshal.SizeOf(statStruct));
+            Marshal.StructureToPtr(statStruct, statStructPtr, true);
+            int result = BlockStats(dom, path, statStructPtr, Marshal.SizeOf(statStruct));
+            Marshal.PtrToStructure(statStructPtr, statStruct);
+            stats = statStruct;
+            Marshal.FreeHGlobal(statStructPtr);
+            return result;
+        }
         /// <summary>
         /// This method will dump the core of a domain on a given file for analysis. Note that for remote Xen Daemon the file path will be interpreted in the remote host.
         /// </summary>
@@ -326,7 +344,35 @@ namespace Libvirt
         /// 0 in case of success or -1 in case of failure.
         /// </returns>
         [DllImport("libvirt-0.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "virDomainInterfaceStats")]
-        public static extern int InterfaceStats(IntPtr dom, string path, DomainInterfaceStatsStruct stats, int size);
+        private static extern int InterfaceStats(IntPtr dom, string path, IntPtr stats, int size);
+        /// <summary>
+        /// This function returns network interface stats for interfaces attached to the domain. 
+        /// The path parameter is the name of the network interface. Domains may have more than one network interface. 
+        /// To get stats for each you should make multiple calls to this function. 
+        /// Individual fields within the stats structure may be returned as -1, 
+        /// which indicates that the hypervisor does not support that particular statistic.
+        /// </summary>
+        /// <param name="dom">
+        /// A <see cref="IntPtr"/>pointer to the domain object.
+        /// </param>
+        /// <param name="path">
+        /// Path to the interface.
+        /// </param>
+        /// <param name="stats">
+        /// A <see cref="DomainInterfaceStatsStruct"/>network interface stats (returned).
+        /// </param>
+        /// <returns>0 in case of success or -1 in case of failure.</returns>
+        public static int InterfaceStats(IntPtr dom, string path, out DomainInterfaceStatsStruct stats)
+        {
+            DomainInterfaceStatsStruct statStruct = new DomainInterfaceStatsStruct();
+            IntPtr statStructPtr = Marshal.AllocHGlobal(Marshal.SizeOf(statStruct));
+            Marshal.StructureToPtr(statStruct, statStructPtr, true);
+            int result = InterfaceStats(dom, path, statStructPtr, Marshal.SizeOf(statStruct));
+            Marshal.PtrToStructure(statStructPtr, statStruct);
+            stats = statStruct;
+            Marshal.FreeHGlobal(statStructPtr);
+            return result;
+        }
         /// <summary>
         /// Determine if the domain is currently running.
         /// </summary>
@@ -421,7 +467,7 @@ namespace Libvirt
 
         // TODO virDomainPinVcpu
 
-        // TODO virDomainMemoryStats
+        // TODO virDomainMemoryStats : Currently disabled in libvirt
 
         /// <summary>
         /// Reboot a domain, the domain object is still usable there after but the domain OS is being stopped for a restart. 
diff --git a/Types.cs b/Types.cs
index 19a36fc..f89bede 100644
--- a/Types.cs
+++ b/Types.cs
@@ -222,27 +222,32 @@ namespace Libvirt
     /// Blocks domain statistics
     ///</summary>
     [StructLayout(LayoutKind.Sequential)]
-    public struct DomainBlockStatsStruct
+    public class DomainBlockStatsStruct
     {
         /// <summary>
         /// Number of read requests.
         /// </summary>
+        [MarshalAs(UnmanagedType.I8)]
         public long rd_req;
         /// <summary>
         /// Number of read bytes.
         /// </summary>
+        [MarshalAs(UnmanagedType.I8)]
         public long rd_bytes;
         /// <summary>
         /// Number of write requests.
         /// </summary>
+        [MarshalAs(UnmanagedType.I8)]
         public long wr_req;
         /// <summary>
         /// Number of written bytes.
         /// </summary>
+        [MarshalAs(UnmanagedType.I8)]
         public long wr_bytes;
         /// <summary>
         /// In Xen this returns the mysterious 'oo_req'.
         /// </summary>
+        [MarshalAs(UnmanagedType.I8)]
         public long errs;
     }
 
@@ -250,39 +255,47 @@ namespace Libvirt
     /// Domain interface statistics
     ///</summary>
     [StructLayout(LayoutKind.Sequential)]
-    public struct DomainInterfaceStatsStruct
+    public class DomainInterfaceStatsStruct
     {
         ///<summary>
         /// Bytes received
         ///</summary>
+        [MarshalAs(UnmanagedType.I8)]
         public long rx_bytes;
         /// <summary>
         /// Packets received
         /// </summary>
+        [MarshalAs(UnmanagedType.I8)]
         public long rx_packets;
         /// <summary>
         /// Errors received
         /// </summary>
+        [MarshalAs(UnmanagedType.I8)]
         public long rx_errs;
         /// <summary>
         /// Drops received
         /// </summary>
+        [MarshalAs(UnmanagedType.I8)]
         public long rx_drop;
         /// <summary>
         /// Bytes sended
         /// </summary>
+        [MarshalAs(UnmanagedType.I8)]
         public long tx_bytes;
         /// <summary>
         /// Packets sended
         /// </summary>
+        [MarshalAs(UnmanagedType.I8)]
         public long tx_packets;
         /// <summary>
         /// Errors sended
         /// </summary>
+        [MarshalAs(UnmanagedType.I8)]
         public long tx_errs;
         /// <summary>
         /// Drops sended
         /// </summary>
+        [MarshalAs(UnmanagedType.I8)]
         public long tx_drop;
     }
 
@@ -477,7 +490,7 @@ namespace Libvirt
     /// Structure for domain memory statistics
     ///</summary>
     [StructLayout(LayoutKind.Sequential)]
-    public struct DomainMemoryStat
+    public class DomainMemoryStat
     {
         /// <summary>
         /// Tag
@@ -545,7 +558,8 @@ namespace Libvirt
     /// <summary>
     /// Structure to handle node informations
     /// </summary>
-    public struct NodeInfo
+    [StructLayout(LayoutKind.Sequential)]
+    public class NodeInfo
     {
         /// <summary>
         /// String indicating the CPU model.
@@ -555,31 +569,38 @@ namespace Libvirt
         /// <summary>
         /// Memory size in kilobytes
         /// </summary>
-        public ulong memory;
+        [MarshalAs(UnmanagedType.SysUInt)]
+        public UIntPtr memory;
         /// <summary>
         /// The number of active CPUs.
         /// </summary>
-        public uint cpus;
+        [MarshalAs(UnmanagedType.SysUInt)]
+        public UIntPtr cpus;
         /// <summary>
         /// Expected CPU frequency.
         /// </summary>
-        public uint mhz;
+        [MarshalAs(UnmanagedType.SysUInt)]
+        public UIntPtr mhz;
         /// <summary>
         /// The number of NUMA cell, 1 for uniform mem access.
         /// </summary>
-        public uint nodes;
+        [MarshalAs(UnmanagedType.SysUInt)]
+        public UIntPtr nodes;
         /// <summary>
         /// Number of CPU socket per node.
         /// </summary>
-        public uint sockets;
+        [MarshalAs(UnmanagedType.SysUInt)]
+        public UIntPtr sockets;
         /// <summary>
         /// Number of core per socket.
         /// </summary>
-        public uint cores;
+        [MarshalAs(UnmanagedType.SysUInt)]
+        public UIntPtr cores;
         /// <summary>
         /// Number of threads per core.
         /// </summary>
-        public uint threads;
+        [MarshalAs(UnmanagedType.SysUInt)]
+        public UIntPtr threads;
     }
     /// <summary>
     /// Structure to handle domain informations
-- 
1.7.2.3.msysgit.0

